package wpd

import (
	"io"
	"os/exec"
)

// Process wraps a wpd process.
type Process interface {
	Stdin() io.WriteCloser
	Stdout() io.ReadCloser
	Stop() error
	Kill() error
}

type Runner func() (Process, error)

// RealProcess runs the actual wpd.
type RealProcess struct {
	cmd    *exec.Cmd
	stdin  io.WriteCloser
	stdout io.ReadCloser
}

// Stdin returns the process input.
func (rr *RealProcess) Stdin() io.WriteCloser {
	return rr.stdin
}

// Stop stops the process.
func (rr *RealProcess) Stop() error {
	rr.stdin.Close()

	return rr.cmd.Wait()
}

func (rr *RealProcess) Kill() error {
	return rr.cmd.Process.Kill()
}

// Stdout returns the process output.
func (rr *RealProcess) Stdout() io.ReadCloser {
	return rr.stdout
}

// NewRealProcess ...
func NewRealProcess() (*RealProcess, error) {
	cmd := exec.Command("wpd")

	in, err := cmd.StdinPipe()
	if err != nil {
		return nil, err
	}

	out, err := cmd.StdoutPipe()
	if err != nil {
		return nil, err
	}

	if err := cmd.Start(); err != nil {
		return nil, err
	}

	return &RealProcess{
		cmd:    cmd,
		stdin:  in,
		stdout: out,
	}, nil
}

// RealProcessRunner ...
func RealProcessRunner() (Process, error) {
	runner, err := NewRealProcess()

	return runner, err
}
