package tests

import (
	"testing"

	"github.com/Masterminds/squirrel"
	"github.com/stretchr/testify/require"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/auth"
	"orus.io/orus-io/rednerd/testutils"
	"orus.io/orus-io/rednerd/testutils/apitester"
)

func TestForgotPassword(t *testing.T) {
	ctx := t.Context()

	tester := apitester.NewAPITester(t, nil)
	defer tester.Close()

	tester.Register(&models.Registration{
		Firstname: "Annie",
		Lastname:  "Ernaux",
		Email:     "example@example.com",
	})

	tester.ForgotPassword(&auth.ForgotPasswordBody{
		Username: "",
		Email:    "example@example.com",
	})

	db := database.NewSQLHelper(ctx, tester.DB, testutils.GetLogger(t))

	var entry models.ResetPasswordTokenDB

	require.NoError(t, db.Get(
		&entry, squirrel.Select("*").From("reset_password_token").
			Where(squirrel.Eq{"username": "example@example.com"}),
	))
}
